{% extends "wagtailadmin/userbar/item_base.html" %}
{% load i18n wagtailadmin_tags %}

{% block item_content %}
    <a href="{{ edit_url|safe }}" target="_self" role="menuitem" id="wagtail-fedit-editor-button">
        <svg xmlns="http://www.w3.org/2000/svg" fill="currentColor" class="w-action-icon" viewBox="0 0 16 16">
            <!-- The MIT License (MIT) -->
            <!-- Copyright (c) 2011-2024 The Bootstrap Authors -->
            <path d="M6 1v3H1V1zM1 0a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h5a1 1 0 0 0 1-1V1a1 1 0 0 0-1-1zm14 12v3h-5v-3zm-5-1a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h5a1 1 0 0 0 1-1v-3a1 1 0 0 0-1-1zM6 8v7H1V8zM1 7a1 1 0 0 0-1 1v7a1 1 0 0 0 1 1h5a1 1 0 0 0 1-1V8a1 1 0 0 0-1-1zm14-6v7h-5V1zm-5-1a1 1 0 0 0-1 1v7a1 1 0 0 0 1 1h5a1 1 0 0 0 1-1V1a1 1 0 0 0-1-1z"/>
        </svg>
        {% trans 'Frontend Editing' %}
    </a>
{% endblock %}