# Generated by Django 5.0.4 on 2024-04-20 22:33

import django.db.models.deletion
import wagtail.blocks
import wagtail.fields
import wagtail.models
import wagtail_fedit.test.core.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0093_uploadedfile'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='BasicModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255)),
                ('body', models.TextField()),
                ('content', wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))])),
            ],
        ),
        migrations.CreateModel(
            name='EditablePageModel',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.page')),
                ('body', models.TextField()),
                ('content', wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))])),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='EditablePreviewModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255)),
                ('body', models.TextField()),
                ('content', wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))])),
            ],
            bases=(wagtail_fedit.test.core.models.BaseEditableMixin, wagtail.models.PreviewableMixin, models.Model),
        ),
        migrations.CreateModel(
            name='EditableDraftModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('live', models.BooleanField(default=True, editable=False, verbose_name='live')),
                ('has_unpublished_changes', models.BooleanField(default=False, editable=False, verbose_name='has unpublished changes')),
                ('first_published_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='first published at')),
                ('last_published_at', models.DateTimeField(editable=False, null=True, verbose_name='last published at')),
                ('go_live_at', models.DateTimeField(blank=True, null=True, verbose_name='go live date/time')),
                ('expire_at', models.DateTimeField(blank=True, null=True, verbose_name='expiry date/time')),
                ('expired', models.BooleanField(default=False, editable=False, verbose_name='expired')),
                ('title', models.CharField(max_length=255)),
                ('body', models.TextField()),
                ('content', wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))])),
                ('latest_revision', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.revision', verbose_name='latest revision')),
                ('live_revision', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.revision', verbose_name='live revision')),
            ],
            options={
                'abstract': False,
            },
            bases=(wagtail_fedit.test.core.models.BaseEditableMixin, models.Model),
        ),
        migrations.CreateModel(
            name='EditableFullModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('live', models.BooleanField(default=True, editable=False, verbose_name='live')),
                ('has_unpublished_changes', models.BooleanField(default=False, editable=False, verbose_name='has unpublished changes')),
                ('first_published_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='first published at')),
                ('last_published_at', models.DateTimeField(editable=False, null=True, verbose_name='last published at')),
                ('go_live_at', models.DateTimeField(blank=True, null=True, verbose_name='go live date/time')),
                ('expire_at', models.DateTimeField(blank=True, null=True, verbose_name='expiry date/time')),
                ('expired', models.BooleanField(default=False, editable=False, verbose_name='expired')),
                ('title', models.CharField(max_length=255)),
                ('body', models.TextField()),
                ('content', wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))])),
                ('latest_revision', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.revision', verbose_name='latest revision')),
                ('live_revision', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.revision', verbose_name='live revision')),
            ],
            options={
                'abstract': False,
            },
            bases=(wagtail_fedit.test.core.models.BaseEditableMixin, models.Model, wagtail.models.WorkflowMixin, wagtail.models.PreviewableMixin),
        ),
        migrations.CreateModel(
            name='EditableLockModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('live', models.BooleanField(default=True, editable=False, verbose_name='live')),
                ('has_unpublished_changes', models.BooleanField(default=False, editable=False, verbose_name='has unpublished changes')),
                ('first_published_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='first published at')),
                ('last_published_at', models.DateTimeField(editable=False, null=True, verbose_name='last published at')),
                ('go_live_at', models.DateTimeField(blank=True, null=True, verbose_name='go live date/time')),
                ('expire_at', models.DateTimeField(blank=True, null=True, verbose_name='expiry date/time')),
                ('expired', models.BooleanField(default=False, editable=False, verbose_name='expired')),
                ('locked', models.BooleanField(default=False, editable=False, verbose_name='locked')),
                ('locked_at', models.DateTimeField(editable=False, null=True, verbose_name='locked at')),
                ('title', models.CharField(max_length=255)),
                ('body', models.TextField()),
                ('content', wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))])),
                ('latest_revision', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.revision', verbose_name='latest revision')),
                ('live_revision', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.revision', verbose_name='live revision')),
                ('locked_by', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='locked_%(class)ss', to=settings.AUTH_USER_MODEL, verbose_name='locked by')),
            ],
            options={
                'abstract': False,
            },
            bases=(wagtail_fedit.test.core.models.BaseEditableMixin, wagtail.models.WorkflowMixin, models.Model),
        ),
        migrations.CreateModel(
            name='EditableRevisionModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255)),
                ('body', models.TextField()),
                ('content', wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))])),
                ('latest_revision', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.revision', verbose_name='latest revision')),
            ],
            options={
                'abstract': False,
            },
            bases=(wagtail_fedit.test.core.models.BaseEditableMixin, models.Model),
        ),
    ]
