.wagtail-fedit-publish-menu {

}
.wagtail-fedit-publish-menu .wagtail-fedit-form-buttons {
    height: 0;
    opacity: 0;
    overflow: hidden;
    transition: height 0.3s;
}
.wagtail-fedit-publish-menu .wagtail-fedit-form-buttons.open {
    height: auto;
    opacity: 1;
}
.wagtail-fedit-userbar-button + .wagtail-fedit-userbar-button.wagtail-fedit-userbar-button {
    margin-inline-start: 0;
}
.wagtail-fedit-userbar-button svg.w-action-icon {
    margin: 0;
    transition: all 0.2s;
}
[name="action-publish"]:hover svg {
    fill: var(--w-color-positive-100);
    opacity: 1;
}
[name="action-unpublish"]:hover svg {
    fill: var(--w-color-critical-200);
    opacity: 1;
}
.wagtail-fedit-userbar-button.initially-hidden {
    display: none;
}
.w-userbar__item a.wagtail-fedit-userbar-button {
    background-color: initial;
    padding-bottom: unset;
    padding-top: unset;
    outline: none;
    border: none;
}
.w-userbar__item a.wagtail-fedit-userbar-button .icon-wrapper {
    background-color: var(--w-color-surface-menu-item-active);
}
.w-userbar__item a.wagtail-fedit-userbar-button:hover {
    background-color: var(--w-color-surface-menu-item-active);
    color: var(--w-color-text-label-menus-default);
}
